package Net::Dropbear::SSH;

use strict;
use v5.8;
our $VERSION = '0.07';

1;
__END__

=encoding utf-8

=head1 NAME

Net::Dropbear - Embed and control a Dropbear SSH client inside of perl (Not implemented yet)

=head1 DESCRIPTION

This module is not implemented yet.

=head1 SEE ALSO

=over

=item L<Net::Dropbear::SSHd> - Embed and control a Dropbear SSH server inside of perl

=item L<Net::OpenSSH>

=back

=head1 AUTHOR

Jon Gentle E<lt>atrodo@cpan.orgE<gt>

=head1 COPYRIGHT

Copyright 2015-2016 Jon Gentle

=head1 LICENSE

This is free software. You may redistribute copies of it under the terms of the Artistic License 2 as published by The Perl Foundation.

=head1 SEE ALSO

=cut
